import math

from OpenGL import GL,GLU

class Camera:
	def __init__(self):
		self.height = math.pi/4 #45
		self.rotate = math.pi/4 + math.pi
		self.dist = 20
	
	def move(self, rel):
		self.rotate -= rel[0]*0.007
		while self.rotate < 0:
			self.rotate += 2*math.pi
		while self.rotate > 2*math.pi:
			self.rotate -= 2*math.pi
		
		self.height += rel[1]*0.007
		if self.height < -0.2*math.pi:
			self.height = -0.2*math.pi
		if self.height > 0.46*math.pi:
			self.height = 0.46*math.pi
	
	def apply(self, lookat):
		up = (0,0,1)
		eyex = self.dist * math.cos(self.height) * math.cos(self.rotate)
		eyey = self.dist * math.cos(self.height) * math.sin(self.rotate)
		eyez = self.dist * math.sin(self.height)
		
		GLU.gluLookAt(
			eyex     ,eyey     ,eyez     ,
			lookat[0],lookat[1],lookat[2],
			up[0]    ,up[1]    ,up[2]    )


	@staticmethod
	def init_3d():
		GL.glEnable(GL.GL_DEPTH_TEST)
		GL.glEnable(GL.GL_COLOR_MATERIAL)
		GL.glEnable(GL.GL_CULL_FACE)
		GL.glEnable(GL.GL_LIGHTING)
		GL.glEnable(GL.GL_LIGHT0)
		GL.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, (0.2,0.2,0.2,1.0))
		GL.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, (0.5,0.5,0.5,1.0))
		GL.glEnable(GL.GL_LIGHT1)
		GL.glLightfv(GL.GL_LIGHT1, GL.GL_DIFFUSE, (0.5,0.5,0.5,1.0))
		#GL.glLightfv(GL. GL_LIGHT0, GL.GL_SPECULAR, (0.0,0.0,0.0,0.0))
		
		GL.glMatrixMode(GL.GL_PROJECTION)
		GL.glLoadIdentity()
		GLU.gluPerspective(45, 1.0, 1, 2000)
		GL.glMatrixMode(GL.GL_MODELVIEW)


	@staticmethod
	def init_2d():
		GL.glDepthMask(0)
		GL.glDisable(GL.GL_DEPTH_TEST)
		
		GL.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE)
		GL.glEnable(GL.GL_BLEND)
		GL.glEnable(GL.GL_TEXTURE_2D)
		GL.glDisable(GL.GL_LIGHTING)
		GL.glBlendFunc(GL.GL_SRC_ALPHA,GL.GL_ONE_MINUS_SRC_ALPHA)
		GL.glColor4f(1.0,1.0,1.0,1.0)
		
		GL.glMatrixMode(GL.GL_PROJECTION)
		GL.glLoadIdentity()
		GL.glMatrixMode(GL.GL_TEXTURE)
		GL.glLoadIdentity()
		GL.glMatrixMode(GL.GL_MODELVIEW)
		GL.glLoadIdentity()
