
class ParserState:
	def __init__(self):
		self.dict = {0:(0,0)} # index: time, value
		self.current_index = 0
	def set_index(self, current_time, index):
		current_index = index
	def update(self, current_time, time, value):
		if time>current_time:
			return
		if time>=self.dict[self.current_index][0]:
			self.dict[self.current_index] = (time, value)

class JointControl:
	def __init__(self, control_string, initial_parameters):
		self.parameters = initial_parameters #{"ParamVel":0,}
		self.commands = []
		
		lines = control_string.to_string().split("\n")
		for line in lines:
			commands = line.split(" ")
			for command in commands:
				self._parse_command(command)
	
	def _parse_command(self, command):
		try:
			if command == "":
				return
			elif command.endswith(":"):
				index = command[:-1]
				index = int(index)
				self.commands.append((ParserState.set_index, (index,)))
			else:
				val_time = command.split("@")
				if len(val_time) != 2:
					raise Exception()
				value = float(val_time[0])
				time = float(val_time[1])
				self.commands.append((ParserState.update, (time, value)))
		except:
			print "Cannot interpret %s" % command
	
	def get_parameters(self, current_time):
		state = ParserState()
		for command in self.commands:
			command[0](state, current_time, *command[1])
		final_dict = {}
		for param_name in self.parameters:
			final_dict[param_name] = state.dict[self.parameters[param_name]][1]
		return final_dict
