import xml.dom.minidom

from OpenGL import GL
import ode

import geometry
import ode_help
import robot_joint

class RobotPart:
	def __init__(self, ode_world, ode_space, part_xml):
		self.ode_body = ode.Body(ode_world)
		self.name = part_xml.getAttribute("Name")
		self.geometries = []
		self.initial_position = eval("[%s]" % part_xml.getAttribute("Position"))
		self.initial_rotation = None
		
		self.ode_body.setPosition(self.initial_position)
		self.initial_rotation = self.ode_body.getRotation()
		
		for geometry_xml in part_xml.getElementsByTagName("Geometry"):
			new_geom = geometry.Geometry.get_geometry(ode_space, geometry_xml)
			new_geom.ode_geom.setBody(self.ode_body)
			self.geometries.append(new_geom)

	def restart(self):
		self.ode_body.setPosition(self.initial_position)
		self.ode_body.setRotation(self.initial_rotation)
		self.ode_body.setLinearVel((0,0,0))
		self.ode_body.setAngularVel((0,0,0))
		
	def draw(self):
		GL.glPushMatrix()
		ode_help.mul_body_matrix(self.ode_body)
		for geometry in self.geometries:
			geometry.draw()
		GL.glPopMatrix()

class Robot:
	def __init__(self, robot_data, ode_world):
		self.parts = []
		self.joints = []
		self.selected_joint = None
		self.ode_world = ode_world
		self.ode_space = ode.Space(type=1) # Create a HashSpace
		
		xmldoc = xml.dom.minidom.parseString(robot_data)
		robot_xml = xmldoc.getElementsByTagName("Robot")[0]
		parts_xml = robot_xml.getElementsByTagName("Parts")[0]
		for part_xml in parts_xml.getElementsByTagName("Part"):
			self.parts.append(RobotPart(self.ode_world, self.ode_space, part_xml))
		joints_xml = robot_xml.getElementsByTagName("Joints")[0]
		for joint_xml in joints_xml.getElementsByTagName("Joint"):
			self.joints.append(robot_joint.RobotJoint.get_robot_joint(self, joint_xml))
	
	def restart(self):
		for part in self.parts:
			part.restart()
		for joint in self.joints:
			joint.restart()
	
	def close(self):
		del self.parts
		del self.joints
		del self.selected_joint
		del self.ode_world
		del self.ode_space
	
	def draw(self):
		for part in self.parts:
			part.draw()
		if None != self.selected_joint:
			self.joints[self.selected_joint].draw()
