import ode
from OpenGL import GL,GLUT

import joint_control
import string_editing

class RobotJoint(object):
	def __init__(self, robot, joint_xml):
		self.ode_joint = None
		self.grip_size = eval(joint_xml.getAttribute("GripSize"))
		self.label = joint_xml.getAttribute("Label")
		self.initial_parameters = {"ParamVel":0,}
		self.joint_control_string = string_editing.StringEditor(joint_xml.getAttribute("InitialControlString"))
		self.joint_control = joint_control.JointControl(self.joint_control_string, self.initial_parameters)
	
	def _attach_bodies(self, robot, joint_xml):
		part1_name = joint_xml.getAttribute("Part1")
		part2_name = joint_xml.getAttribute("Part2")
		for part in robot.parts:
			if part.name == part1_name:
				part1 = part
			if part.name == part2_name:
				part2 = part
		self.ode_joint.attach(part1.ode_body, part2.ode_body)
	
	def draw(self):
		GL.glColor4f(0,1,0,0.4)
		GL.glPushMatrix()
		GL.glTranslatef(*self.ode_joint.getAnchor())
		GLUT.glutSolidSphere(self.grip_size,10,10)
		GL.glPopMatrix()
	
	def restart(self):
		self.joint_control = joint_control.JointControl(self.joint_control_string, self.initial_parameters)
	
	def set_params(self, simulation_time):
		params_dict = self.joint_control.get_parameters(simulation_time)
		for param_name in params_dict:
			parameter = getattr(ode, param_name)
			self.ode_joint.setParam(parameter, params_dict[param_name])
	
	@staticmethod
	def get_robot_joint(robot, joint_xml):
		type = joint_xml.getAttribute("Type")
		if "Hinge" == type:
			return RobotHingeJoint(robot, joint_xml)
		raise Exception("Unrecognized joint type %s" % type)

class RobotHingeJoint(RobotJoint):
	def __init__(self, robot, joint_xml):
		super(RobotHingeJoint, self).__init__(robot, joint_xml)
		self.ode_joint = ode.HingeJoint(robot.ode_world)
		
		self._attach_bodies(robot, joint_xml)
		
		anchor = eval("[%s]" % joint_xml.getAttribute("Anchor"))
		self.ode_joint.setAnchor(anchor)
		axis = eval("[%s]" % joint_xml.getAttribute("Axis"))
		self.ode_joint.setAxis(axis)
		
		for param_xml in joint_xml.getElementsByTagName("Parameter"):
			parameter = getattr(ode, param_xml.getAttribute("Name"))
			self.ode_joint.setParam(parameter, eval(param_xml.getAttribute("Value")))
		
