import pygame
from OpenGL import GL

import font

class StringEditor:
	def __init__(self, initial_string):
		self.lines = initial_string.split("\n")
		self.cursor_line = 0
		self.cursor_column = 0
	
	def to_string(self):
		return "\n".join(self.lines)
	
	def draw(self):
		GL.glColor3f(0,0,0)
		GL.glPushMatrix()
		if self.cursor_line - 3 >=0:
			GL.glBegin(GL.GL_TRIANGLE_STRIP)
			GL.glVertex3f(14,-2,0)
			GL.glVertex3f(6,5,0)
			GL.glVertex3f(22,5,0)
			GL.glEnd()
		if self.cursor_line - 2 >=0:
			font.Font.get_font().draw(self.lines[self.cursor_line - 2])
		GL.glTranslatef(0,18,0)
		if self.cursor_line - 1 >=0:
			font.Font.get_font().draw(self.lines[self.cursor_line - 1])
		GL.glTranslatef(0,18,0)
		
		font.Font.get_font().draw(self.lines[self.cursor_line])
		GL.glBegin(GL.GL_LINE_STRIP)
		GL.glVertex(12*self.cursor_column+1,4,0)
		GL.glVertex(12*self.cursor_column+1,24,0)
		GL.glEnd()
		
		GL.glTranslatef(0,18,0)
		if self.cursor_line + 1 < len(self.lines):
			font.Font.get_font().draw(self.lines[self.cursor_line + 1])
		GL.glTranslatef(0,18,0)
		if self.cursor_line + 2 < len(self.lines):
			font.Font.get_font().draw(self.lines[self.cursor_line + 2])
		if self.cursor_line + 3 < len(self.lines):
			GL.glBegin(GL.GL_TRIANGLE_STRIP)
			GL.glVertex3f(22,22,0)
			GL.glVertex3f(6,22,0)
			GL.glVertex3f(14,29,0)
			GL.glEnd()
		GL.glPopMatrix()
	
	def _fix_cursor_in_line(self):
		if self.cursor_column > len(self.lines[self.cursor_line]):
			self.cursor_column = len(self.lines[self.cursor_line])
	
	def _add_char(self, text):
		current_line = self.lines[self.cursor_line]
		current_line = current_line[:self.cursor_column] + text + current_line[self.cursor_column:]
		self.lines[self.cursor_line] = current_line
		self.cursor_column += len(text)
	
	def _add_newline(self):
		newline = self.lines[self.cursor_line][self.cursor_column:]
		self.lines[self.cursor_line] = self.lines[self.cursor_line][:self.cursor_column]
		self.cursor_column = 0
		self.cursor_line += 1
		self.lines.insert(self.cursor_line,newline)
	
	def _backspace(self):
		if self.cursor_column > 0:
			self.cursor_column -= 1
			current_line = self.lines[self.cursor_line]
			current_line = current_line[:self.cursor_column] + current_line[self.cursor_column+1:]
			self.lines[self.cursor_line] = current_line
		elif self.cursor_line > 0:
			self.cursor_line -= 1
			self.cursor_column = len(self.lines[self.cursor_line])
			self.lines[self.cursor_line] = self.lines[self.cursor_line]+self.lines[self.cursor_line+1]
			self.lines = self.lines[:self.cursor_line+1] + self.lines[self.cursor_line+2:]
	
	def _right(self):
		if self.cursor_column > len(self.lines[self.cursor_line]) - 1:
			if self.cursor_line < len(self.lines) - 1:
				self.cursor_column = 0
				self.cursor_line += 1
		else:
			self.cursor_column += 1
	
	def _left(self):
		if self.cursor_column == 0:
			if self.cursor_line > 0:
				self.cursor_line -= 1
				self.cursor_column = len(self.lines[self.cursor_line])
		else:
			self.cursor_column -= 1

	def process_key(self, key, char):
		if key == pygame.K_UP:
			if self.cursor_line > 0:
				self.cursor_line -= 1
			self._fix_cursor_in_line()
		elif key == pygame.K_DOWN:
			if self.cursor_line < len(self.lines) - 1:
				self.cursor_line += 1
			self._fix_cursor_in_line()
		elif key == pygame.K_RIGHT:
			self._right()
		elif key == pygame.K_LEFT:
			self._left()
		elif key == pygame.K_RETURN:
			self._add_newline()
		elif key == pygame.K_DELETE:
			self._right()
			self._backspace()
		elif key == pygame.K_HOME:
			self.cursor_column = 0
		elif key == pygame.K_END:
			self.cursor_column = len(self.lines[self.cursor_line])
		elif key == pygame.K_BACKSPACE:
			self._backspace()
		elif key >= pygame.K_a and key <= pygame.K_z:
			self._add_char(char)
		elif key >= pygame.K_0 and key <= pygame.K_9:
			self._add_char(char)
		elif key in [pygame.K_SPACE, pygame.K_MINUS, pygame.K_PERIOD, pygame.K_SEMICOLON]:
			self._add_char(char)
		